package models

import (
	"context"
	"testing"

	"github.com/Masterminds/squirrel"
	"github.com/jackc/pgtype"
	"github.com/stretchr/testify/require"
	"gotest.tools/assert"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/migration"
	"orus.io/orus-io/rednerd/testutils"
)

func TestTemplateDB(t *testing.T) {
	log := testutils.GetLogger(t)
	ctx := log.WithContext(context.Background())
	db := database.GetTestDB(ctx, t, migration.Source)
	defer db.Close()

	_, err := database.Exec(db, database.SQLInsert(AccountDB{Name: "account"}), &log)
	require.NoError(t, err)

	_, err = database.Exec(db, database.SQLInsert(
		TemplateDB{
			Template: Template{
				Account:        "account",
				Name:           "mytemplate",
				StaticMetadata: Metadata{"title": "hi"},
				Metadata:       Metadata{"title": "hi {{coucou}}"},
			},
			InternalVersion: 1,
			BytesBody: pgtype.Bytea{
				Bytes:  []byte("body"),
				Status: pgtype.Present,
			},
		}), &log)
	require.NoError(t, err)

	var template TemplateDB
	require.NoError(t, database.Get(
		db,
		&template,
		squirrel.Select(template.Columns(true)...).
			From(template.Table()),
		&log))
	assert.Equal(t, "hi", template.StaticMetadata["title"])
	assert.Equal(t, "hi {{coucou}}", template.Metadata["title"])
}
