package handlers

import (
	"context"
	"testing"
	"time"

	"github.com/stretchr/testify/require"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/migration"
	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/testutils"
)

func GetUser(t *testing.T, fname string, lname string) (*models.Userinfo, string) {
	t.Helper()

	return &models.Userinfo{
		Email:     fname + "." + lname + "@test.fr",
		Firstname: fname,
		Lastname:  lname,
		Username:  fname + lname,
		IsAdmin:   false,
	}, fname + lname
}

func AddUserActivity(t *testing.T, username string, qtyCurrentMonth int, qtyLastMonth int) []*models.RenderLog {
	t.Helper()
	var renderList []*models.RenderLog
	for k := 0; k < qtyCurrentMonth; k++ {
		rd := models.NewRenderLog(time.Now(), "text", "mjml", username)
		rd.SetDuration(time.Second)
		renderList = append(renderList, rd)
	}
	for k := 0; k < qtyLastMonth; k++ {
		rd := models.NewRenderLog(time.Now().AddDate(0, -1, 0), "text", "mjml", username)
		rd.SetDuration(time.Second)
		renderList = append(renderList, rd)
	}

	return renderList
}

func TestUserGetHandler(t *testing.T) {
	log := testutils.GetLogger(t)
	ctx := log.WithContext(context.Background())
	db := database.GetTestDB(ctx, t, migration.Source)
	defer db.Close()

	// creating the user using the add handler
	addHandler := NewUserAddHandler(db.DB)

	user, username := GetUser(t, "John", "Doe")
	_, err := addHandler.handle(ctx, user)
	// not checking if the user has been properly created, we trust user-add
	require.NoError(t, err)

	// adding some dummy activity for the user JohnDoe
	qtyCurrentMonth := 4
	qtyLastMonth := 2
	renderList := AddUserActivity(t, username, qtyCurrentMonth, qtyLastMonth)
	for _, rlog := range renderList {
		_, err = database.Exec(db, database.SQLInsert(rlog), &log)
		require.NoError(t, err)
	}

	t.Run("basic", func(t *testing.T) {
		getHandler := NewUserGetHandler(db.DB)
		user, err := getHandler.handle(ctx, username)
		require.NoError(t, err)
		stats := user.Activity.RenderStats
		expectedTotal := qtyCurrentMonth + qtyLastMonth
		require.Equal(t, int64(expectedTotal), stats.Total)
		require.Equal(t, int64(qtyCurrentMonth), stats.CurrentMonth)
		require.Equal(t, int64(qtyLastMonth), stats.LastMonth)
	})
}
