package tests

import (
	"encoding/base64"
	"fmt"
	"os"
	"strings"
	"sync"
	"sync/atomic"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/cmd/rednerd/cmd"
	"orus.io/orus-io/rednerd/engines/wpd"
	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi"
)

func TestWPDHeavyLoad(t *testing.T) {
	engineOptions := cmd.NewEngineOptions()

	require.NoError(t, engineOptions.SetupEngineOptions(nil))
	engineOptions.SetOptions(
		"WPD", &wpd.EngineOptions{
			WorkerPoolSize: 10,
		})
	tester := NewAPITester(t, &engineOptions, func(s *restapi.Server) {
		s.WriteTimeout = time.Minute * 5
	})
	defer tester.Close()

	defer tester.Logout()
	tester.Login("admin", "admin")

	tester.CreateUser("janedoe", "janedoe")
	tester.Login("janedoe", "janedoe")

	var wg sync.WaitGroup
	var count uint64
	renderCount := 1000
	if testing.Short() {
		renderCount = 20
	}
	for i := 0; i != renderCount; i++ {
		wg.Add(1)
		go func(i int) {
			defer wg.Done()
			var result []models.Document
			body := "<html><body><h1>Lorem Ipsum</h1>" + strings.Repeat("<p>"+loremIpsum+"</p>", i/10+1) + "</body></html>"
			require.Equal(t, 200, tester.MustPostJSON(
				"/api/v1/render",
				JSONObj{
					"accept": "application/pdf",
					"document": JSONObj{
						"type":        "text/html",
						"body-format": "text",
						"body":        body,
					},
				},
				&result,
			))
			require.Equal(t, 1, len(result))
			if body, err := base64.StdEncoding.DecodeString(result[0].Body); assert.NoError(t, err) && assert.NotZero(t, len(body)) {
				if i < 10 {
					require.NoError(t, os.WriteFile(fmt.Sprintf("wpd-heavy-load-%.3d.pdf", i), body, 0o600))
				}
			}
			t.Logf("%d/1000: Rendered doc n°%d", atomic.AddUint64(&count, 1), i)
		}(i)
	}
	wg.Wait()
}

var loremIpsum = `Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.`
